"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TestContext = void 0;
const ts = __importStar(require("@terascope/utils"));
const nanoid_1 = require("nanoid");
const core_context_1 = require("./core-context");
const validate_configs_1 = __importDefault(require("./validate-configs"));
function getKey(opts) {
    const { type, endpoint = 'default' } = opts;
    if (!ts.isString(type))
        throw new Error('A type must be specified when registering a Client');
    return `${type}:${endpoint}`;
}
function setConnectorConfig(sysconfig, opts, config, override = true) {
    const { type, endpoint = 'default' } = opts;
    const { connectors } = sysconfig.terafoundation;
    if (connectors[type] == null)
        connectors[type] = {};
    if (connectors[type][endpoint] == null) {
        connectors[type][endpoint] = config;
    }
    else if (override) {
        connectors[type][endpoint] = config;
    }
    return connectors[type][endpoint];
}
const _cachedClients = new WeakMap();
const _createClientFns = new WeakMap();
class TestContext extends core_context_1.CoreContext {
    constructor(options = {}) {
        const sysconfig = {
            terafoundation: {
                connectors: {
                    elasticsearch: {
                        default: {},
                    },
                },
                ...ts.get(options.sysconfig, 'terafoundation', {})
            },
            ...options.sysconfig
        };
        const config = {
            name: options.name || 'test-context',
        };
        const cluster = {
            isMaster: false,
            worker: {
                id: (0, nanoid_1.nanoid)(8),
            }
        };
        super(config, cluster, (0, validate_configs_1.default)(cluster, config, sysconfig));
        // eslint-disable-next-line @typescript-eslint/no-this-alias
        const ctx = this;
        _cachedClients.set(this, {});
        _createClientFns.set(this, {});
        this.apis.foundation.getConnection = (opts) => {
            const { cached } = opts;
            const cachedClients = _cachedClients.get(ctx) || {};
            const key = getKey(opts);
            if (cached && cachedClients[key] != null) {
                return cachedClients[key];
            }
            const clientFns = _createClientFns.get(ctx) || {};
            const create = clientFns[key];
            if (!create)
                throw new Error(`No client was found for connection "${key}"`);
            if (!ts.isFunction(create)) {
                const actual = ts.getTypeOf(create);
                throw new Error(`Registered Client for connection "${key}" is not a function, got ${actual}`);
            }
            const connectorConfig = setConnectorConfig(ctx.sysconfig, opts, {}, false);
            const client = create(connectorConfig, ctx.logger, opts);
            cachedClients[key] = client;
            _cachedClients.set(ctx, cachedClients);
            return client;
        };
        this.apis.setTestClients = (clients = []) => {
            clients.forEach((clientConfig) => {
                const { create, config: connectionConfig = {} } = clientConfig;
                const clientFns = _createClientFns.get(ctx) || {};
                const key = getKey(clientConfig);
                if (!ts.isFunction(create)) {
                    const actual = ts.getTypeOf(create);
                    throw new Error(`Test Client for connection "${key}" is not a function, got ${actual}`);
                }
                ctx.logger.trace(`Setting test client for connection "${key}"`, connectionConfig);
                clientFns[key] = create;
                _createClientFns.set(ctx, clientFns);
                const cachedClients = _cachedClients.get(ctx) || {};
                delete cachedClients[key];
                _cachedClients.set(ctx, cachedClients);
                setConnectorConfig(ctx.sysconfig, clientConfig, connectionConfig, true);
            });
        };
        this.apis.getTestClients = () => {
            const cachedClients = _cachedClients.get(ctx) || {};
            const clients = {};
            Object.keys(cachedClients).forEach((key) => {
                const [type, endpoint] = key.split(':', 2);
                if (clients[type] == null) {
                    clients[type] = {};
                }
                clients[type][endpoint] = cachedClients[key];
            });
            return clients;
        };
        this.foundation.getConnection = this.apis.foundation.getConnection;
        if (options.clients) {
            this.apis.setTestClients(options.clients);
        }
    }
}
exports.TestContext = TestContext;
//# sourceMappingURL=test-context.js.map